//
// PovRay scene file for a sci-fi laser cannon
//
// Author:   Hj. Malthaner
// Web:      http://opengameart.org/users/varkalandar
//
// This file is available under the GNU general
// public license v2 or newer.
//
// http://www.gnu.org/licenses/gpl-2.0.html
//

                                                   
#version 3.6;


#declare ACTIVE_WEAPONS = 0;


global_settings 
{
  assumed_gamma 2.2
}

                                                   
#local TEX_LASER_GOLD =
texture
{
  pigment 
  {
    bozo
    color_map
    {
      [0.0 color srgb <0.1, 0.05, 0.02>]
      [0.2 color srgb <0.5, 0.2, 0>]
      [1.0 color srgb <1.0, 0.92, 0.65>]
    }  
    scale 1.5  
	quick_color srgb 1
  }
  normal
  {
    bumps 0.2
    scale <0.1, 0.5, 0.1>
  }
  finish
  {
    ambient 0.1
    emission 0.0
    diffuse 0.7
    specular 0.3
    roughness 0.1
  
    metallic
    reflection
    {
      0.05
      metallic
    }
  }
}


#local TEX_LASER_CHROME =
texture
{
  pigment 
  {
    bozo
    color_map
    {
      [0.1 color srgb <0.3, 0.4, 0.9> ]
      [0.5 color srgb <0.7, 0.8, 0.9> ]
      [1.0 color srgb <0.8, 0.9, 1.0> ]
    }        
    turbulence 0.2
    scale 1.0  
	quick_color srgb 1
  }
  finish
  {
    ambient 0.1
    emission 0.0
    diffuse 0.7
    specular 0.4
    roughness 0.07
  
    metallic
    reflection
    {
      0.15
      metallic
    }
    brilliance 3
  }
}


#local TEX_FADING_CHROME =
texture
{
  pigment 
  {
    gradient y
    color_map
    {
      [0.0 color srgbf <1.0, 1.0, 1.0, 1> ]
      [1.0 color srgbf <0.7, 0.45, 0.3, 0.7> ]
    }        
    turbulence 0.2
    scale 1.0  
    quick_color srgb 1
  }
  finish
  {
    ambient 0.1
    emission 0.0
    diffuse 0.7
    specular 0.4
    roughness 0.07
  
    metallic
    reflection
    {
      0.15
      metallic
    }
    brilliance 3
  }
}


// simplified glass for faster renders
#local COLOR_GLASS = <0.95, 0.97, 0.99>;
#local COLOR_GLASS_TRANSP = COLOR_GLASS + <0, 0, 0, 0.8, 0.2>;
    
#local TEX_LASER_GLASS =
texture
{
  pigment 
  {
    color srgbft COLOR_GLASS_TRANSP
		quick_color srgb 1
  }
  finish
  {
    ambient 0.0
    emission 0.2
    diffuse 0.0
    specular 0.2
    roughness 0.05
  }
}


#macro O_QUARTER_RING(MAJOR, MINOR)
intersection
{
  torus
  {
    MAJOR, MINOR
  }
  box
  {
    <0, -MINOR-0.001, 0>
    <MAJOR+MINOR, +MINOR+0.001, MAJOR+MINOR>
  }
}
#end


#macro O_HALF_RING(OUTER, INNER)
intersection
{
  torus
  {
    OUTER-INNER, INNER
  }
  box
  {
    <-OUTER-0.001, -INNER-0.001, -OUTER-0.001>
    <0, +INNER+0.001, +OUTER+0.001>
  }
}
#end


#macro O_RING(OUTER, INNER, LENGTH)
union
{
  O_HALF_RING(OUTER, INNER)
  
  object
  {  
    O_HALF_RING(OUTER, INNER)
    rotate <0, 180, 0> 
    translate <LENGTH, 0, 0>
  }
  
  cylinder
  {
    <0, 0, -OUTER+INNER>, <LENGTH, 0, -OUTER+INNER>, INNER
  }
  
  cylinder
  {
    <0, 0, +OUTER-INNER>, <LENGTH, 0, +OUTER-INNER>, INNER
  }
}
#end


#macro O_ROD(RADIUS, LENGTH)
merge
{
  sphere
  {
    0, RADIUS
  }
  sphere
  {
    <0, LENGTH, 0>, RADIUS
  }
  cylinder
  {
    <0, 0, 0>, <0, LENGTH, 0>, RADIUS
  }
}
#end


#macro O_SOFT_PIPE(OUTER, INNER, LENGTH)
union
{
  #local THICKNESS = OUTER-INNER;
  torus
  {
    OUTER-THICKNESS/2, THICKNESS/2
  }
  torus
  {
    OUTER-THICKNESS/2, THICKNESS/2
    translate <0, LENGTH, 0>
  }
  difference
  {
    cylinder
    {
      <0, 0, 0>, <0, LENGTH, 0>, OUTER
    }
    cylinder
    {
      <0, -0.001, 0>, <0, LENGTH+0.001, 0>, INNER
    }
  }
}
#end


#local O_LASER_BARREL =
union
{
  difference
  {
    object
    {
      O_SOFT_PIPE(1.5, 1.3, 14)
      translate <0, 2.5, 0>
    }
  
    #local row = 0;
    #local row_angle = 0;
    
    #while(row < 10)
      #local n = 0;
      #while(n < 3)
        cylinder
        {     
          <-1.51, 0, 0>, <1.51, 0, 0>, 0.4
          
          translate <0, 5 + row*1.0, 0>
          
          rotate <0, n*360/6 + row_angle, 0>
        }
        #local n = n + 1;
      #end
      
      #local row = row + 1;
      
      #if(row_angle < 360/12)
        #local row_angle = 360/12;
      #else
        #local row_angle = 0;
      #end
      
    #end
    
    texture
    {
      TEX_LASER_GOLD
    }
  }
  
  // mount
  torus
  {
    1.5, 0.1
    scale <1, 3, 1>
    translate <0, 3, 0>
  }
  torus
  {
    1.5, 0.1
    scale <1, 3, 1>
    translate <0, 15, 0>
  }

  // side parts to hold the barrel
  union
  {
    superellipsoid
    {
      <0.1, 0.1>
      scale <0.2, 6.5, 0.6>
      translate <-1.7, 9, 0>
    }  
    superellipsoid
    {
      <0.1, 0.1>
      scale <0.2, 6.5, 0.6>
      translate <+1.7, 9, 0>
    }
    superellipsoid
    {
      <0.1, 0.1>
      scale <0.1, 6.3, 0.4>
      translate <-1.9, 9.0, 0>
    }  
    superellipsoid
    {
      <0.1, 0.1>
      scale <0.1, 6.3, 0.4>
      translate <+1.9, 9.0, 0>
    }
    texture { TEX_LASER_CHROME }    
  }

  // bolts for the mount
  #local n = 0;
  #while(n < 16)
    sphere
    {     
      <0, 0, 0>, 0.2
      scale <1, 1, 0.3>
      translate <0, 3, 1.6>
      rotate <0, n*360/16, 0>
    }
    sphere
    {     
      <0, 0, 0>, 0.2
      scale <1, 1, 0.3>
      translate <0, 15, 1.6>
      rotate <0, n*360/16, 0>
    }
    #local n = n + 1;
  #end
  

  // pipe holder
  difference
  {
    union
    {
      cylinder
      {
        <0, 3-0.3, 0>, <0, 3+0.3, 0>, 1.5
      }
      cylinder
      {
        <0, 15-0.3, 0>, <0, 15+0.3, 0>, 1.5
      }
    }
    cylinder
    {
      <0, 2, 0>, <0, 16, 0>, 1.0
    }
  }
    
  #local O_PUMP = O_ROD(0.15, 16)

  union
  {
    // pumps
    #local n = 0;
    #while(n < 8)
      object
      {     
        O_PUMP
        
        translate <0, 1.5, 1.2>
        
        rotate <0, n*360/8, 0>
      }
      #local n = n + 1;
    #end
             
             
    #local O_GUIDE = O_ROD(0.15, 22)

    
    // focus guides
    #local n = 0;
    #while(n < 8)
      object
      {     
        O_GUIDE
        
        translate <0, 0, 1.2>
        
        rotate <0, n*360/8 + 360/16, 0>
      }
      #local n = n + 1;
    #end
               
    #local O_ARC = O_QUARTER_RING(1, 0.15)
    
    difference
    {
      union
      {
        // focus arches
        #local n = 0;
        #while(n < 8)
          object
          {     
            O_ARC
            rotate <0, 0, 90>
            scale <1, 3, 1>
            translate <0, 22, 0.2>
            
            rotate <0, n*360/8 + 360/16, 0>
          }
          sphere
          {
            <0, 22, 1.2>, 0.15
            rotate <0, n*360/8 + 360/16, 0>
          }
          #local n = n + 1;
        #end
      }
      cylinder
      {
        <0, 22, 0>, <0, 25.5, 0>, 0.3
      }    
      texture
      {
        TEX_LASER_CHROME
      }
      texture
      {
        TEX_FADING_CHROME
        scale 4
        translate <0, 21.8, 0>
      }
    }

    torus
    {
      0.3, 0.1
      scale <1, 4, 1>
      translate <0, 25.0, 0>
      texture
      {
        TEX_LASER_CHROME
      }
      texture
      {
        TEX_FADING_CHROME
        scale 4
        translate <0, 21.8, 0>
      }
    }           
    
    // rear reflector
    union
    {
      cylinder
      {
        <0, 0.0, 0>, <0, 0.3, 0>, 1.4
      }    
      sphere
      {
        0, 1
        scale <1, 0.2, 1>      
      }
      texture
      {
        TEX_LASER_CHROME
      }
      texture
      {
        TEX_FADING_CHROME
        scale <1, -1, 1>
        translate <0, 0.5, 0>
      }
    }           
    
    
    // power wires
    
    
    #local n = 0;
    #while(n < 5)
      torus
      {
        0.6, 0.2
        translate <-1.1, 5+n*2, -0.8>
      }
      torus
      {
        0.6, 0.2
        translate <+1.1, 5+n*2, -0.8>
      }
      #local n = n + 1;
    #end
    
    texture
    {
      TEX_LASER_CHROME
    }
  }  

  // laser tube
  union
  {
    difference
    {
      object
      {
        O_ROD(1, 22)
        translate <0, 1.0, 0>
      }
      object
      {
        O_ROD(0.85, 21.7)
        translate <0, 1.15, 0>
      }
    }
    object
    {
      O_ROD(0.55, 21.8)
      translate <0, 1.1, 0>
    }
    object
    {
      O_ROD(0.4, 22)
      translate <0, 1, 0>
      texture
      {
        TEX_LASER_GLASS
        pigment 
        { 
          agate
          color_map
          {
            [0.6 color srgbft <0.0, 0.05, 0.1, 0.4, 0.6>]
            [1.0 color srgbft <2.8, 2.9, 3.5, 0.0, 0.0>]
          }
          
          scale <0.3, 1.2, 0.3>
        }        
      }
      
      interior
      {
        ior 2.5
        fade_color <0.5, 0.2, 0.5>
        fade_distance 0.2
        fade_power 1001
      }
    }
    
    texture
    {
      TEX_LASER_GLASS
    }
    
    interior
    {
      ior 1.5
      fade_color COLOR_GLASS
      fade_distance 0.2
      fade_power 1001
    }
  }
  
#if(ACTIVE_WEAPONS)  

  // Testing lightning aura
  sphere
  {
    <0, 14, 0>, 18
    scale <0.5, 1, 0.5>
    
    pigment { rgbt 1 } 
    hollow

    interior
    {
      
      media
      { 
        emission 2.5
        
        density
        { 
          spherical
          warp
          {
            turbulence 0.5 octaves 3 omega 0.5 lambda 2.0
          }
          scale <0.4, 1, 0.4> * 16
          translate <0, 16, 0>
          rotate <0, 90, 0>
        }
        
        density
        { 
          crackle
          density_map
          { 
            [0.01  rgb <1.0, 1.0, 1.0>]  // outer
            [0.025  rgb <0.5, 0.6, 1.0>]
            [0.04  rgb <0.05, 0.02, 0.2>]
            [0.50  rgb <0.0, 0.0, 0.1>]
            [0.80  rgb <0.0, 0.0, 0.0>]  // inner
          }
          scale <2, 2, 2>
          warp
          {
            turbulence 0.5 octaves 5 omega 0.5 lambda 2.5
          }
        }
      }
    }
    
  }
  
  // expensive lights ...
  #local n = 0;
  #while(n < 12)
    light_source
    {
      <0, 1 + n*2, 0>
      // standby light
      color srgb <0.05, 0.1, 0.5>
      // shooting light
      // color srgb <0.1, 0.1, 0.5>*8
      fade_power 2
      fade_distance 0.4
    }
    #local n = n + 1;
  #end

  // shot - beam
  union
  {
    cylinder
    {
      <0, 20, 0>, <0, 400, 0>, 0.28
      no_shadow
      
      texture
      {
        pigment { color srgb <0.8, 0.9, 1.0> }
        finish { ambient 1.0 }
      }
    }

    light_source  
    {
      <0, 27, 0>
      color srgb <0.8, 0.9, 1.0>*2
      fade_distance 1.6
      fade_power 3
    }
  }
#end
}

#local O_LASER_TURRET =
union
{
  cylinder
  {
    <0, 0, 0>, <0, -0.2, 0>, 5.5
  }
  
  cylinder
  {
    <0, -0.18, 0>, <0, -0.21, 0>, 5.9
    texture { TEX_LASER_CHROME pigment {color srgb <0.5, 0.6, 0.7>} }
  }

  // fixed base with soft rim
  cylinder
  {
    <0, -0.2, 0>, <0, -1.2, 0>, 7
  }
  intersection
  {
    torus
    {
      7, 0.1
      scale <1, 10, 1>
      translate <0, -1, 0>
    }  
    cylinder
    {
      <0, -0.2, 0>, <0, -1.2, 0>, 7.1
    }
  }  
  
  
  difference
  // union
  {
    box 
    {
      <-3, 0, 4,>
      <3, 2, 10>
    }
    
    box 
    {
      <-2, 0, 0>
      <2, 2, -7>
      
      rotate <10, 0, 0>
      translate <0, 1.1, 10>
    }
    translate <0, 0, -7>
  }
  
  // big side mounts
  cylinder
  {
    <0, 0, 0>, <0.9, 0, 0>, 3
    scale <1, 2, 1>
    translate <-3.1, 2, 0>
  }
  cylinder
  {
    <0, 0, 0>, <-0.9, 0, 0>, 3
    scale <1, 2, 1>
    translate <+3.1, 2, 0>
  }
  
  cylinder
  {
    <0, 0, 0>, <1, 0, 0>, 2.7
    scale <1, 2, 1>
    translate <-3.3, 1, 0>
  }
  cylinder
  {
    <0, 0, 0>, <-1, 0, 0>, 2.7
    scale <1, 2, 1>
    translate <+3.3, 1, 0>
  }

  cylinder
  {
    <0, 0, 0>, <1, 0, 0>, 2.5
    scale <1, 1.6, 1>
    translate <-3.8, 1, 0>
  }
  cylinder
  {
    <0, 0, 0>, <-1, 0, 0>, 2.5
    scale <1, 1.6, 1>
    translate <+3.8, 1, 0>
  }

  cylinder
  {
    <0, 0, 0>, <1, 0, 0>, 2.4
    scale <1, 1.3, 1>
    translate <-4.3, 0.9, 0>
  }
  cylinder
  {
    <0, 0, 0>, <-1, 0, 0>, 2.4
    scale <1, 1.3, 1>
    translate <+4.3, 0.9, 0>
  }

  cylinder
  {
    <0, 0, 0>, <1, 0, 0>, 2.2
    scale <1, 1.2, 1>
    translate <-4.8, 0.4, 0>
  }
  cylinder
  {
    <0, 0, 0>, <-1, 0, 0>, 2.2
    scale <1, 1.2, 1>
    translate <+4.8, 0.4, 0>
  }

  // axle
  union
  {
    cylinder
    {
      <-2.1, 5.9, 0>, <-1.7, 5.9, 0>, 1.9
    }
    cylinder
    {
      <1.7, 5.9, 0>, <2.1, 5.9, 0>, 1.9
    }
    texture { TEX_LASER_CHROME }
  }
}

#local O_LASER_TURRET_GIRDER =
union
{
  cylinder
  {
    <0, 0, 0>, <0, -0.2, 0>, 5.5
  }
  
  cylinder
  {
    <0, -0.18, 0>, <0, -0.21, 0>, 5.9
    texture { TEX_LASER_CHROME pigment {color srgb <0.5, 0.6, 0.7>} }
  }

  // fixed base with soft rim
  cylinder
  {
    <0, -0.2, 0>, <0, -1.2, 0>, 7
  }
  intersection
  {
    torus
    {
      7, 0.1
      scale <1, 10, 1>
      translate <0, -1, 0>
    }  
    cylinder
    {
      <0, -0.2, 0>, <0, -1.2, 0>, 7.1
    }
  }  
  
  
  union
  {
    difference
    {
      box 
      {
        <-3, 0, 3.5,>
        <3, 1, 10.5>
      }
      
      box 
      {
        <-2, 0, 0>
        <2, 1, -7>
        
        rotate <10, 0, 0>
        translate <0, 0.1, 10>
      }
    }
      
    // round sides
    cylinder 
    {
      <-3, 0, 3.5>, <-3, 0, 10.5>, 1.0
    }
    cylinder 
    {
      <+3, 0, 3.5>, <+3, 0, 10.5>, 1.0
    }
    
    translate <0, 0, -7>
    texture { TEX_LASER_CHROME }
  }
  
  // mounts
  
  union
  {
    // left side
    cylinder 
    {
      <-2.5, 0.7, 2.66>, <-2.5, 5.9, 0>, 0.4
    }
    cylinder 
    {
      <-2.5, 0.7, -2.66>, <-2.5, 5.9, 0>, 0.4
    }
    
    // right side
    cylinder 
    {
      <+2.5, 0.7, 2.66>, <+2.5, 5.9, 0>, 0.4
    }
    cylinder 
    {
      <+2.5, 0.7, -2.66>, <+2.5, 5.9, 0>, 0.4
    }
    
    // axle
    cylinder
    {
      <-2.9, 5.9, 0>, <-2.0, 5.9, 0>, 0.7
    }
    cylinder
    {
      <2.9, 5.9, 0>, <2.0, 5.9, 0>, 0.7
    }
    
    // guidance disks
    cylinder
    {
      <-2.0, 5.9, 0>, <-1.7, 5.9, 0>, 1.9
    }
    cylinder
    {
      <1.7, 5.9, 0>, <2.0, 5.9, 0>, 1.9
    }
    
    // aiming torus
    torus
    {
      1.85, 0.15
      scale <1, 5, 1>
      rotate <90, 0, 0>
      rotate <20, 0, 0>
      translate <0, 6.8, 0.4>
    }
    
    
    texture { TEX_LASER_CHROME }
  }
}

#declare O_LASER_CANNON =
object
{
  union
  {
    object
    {
      O_LASER_BARREL
      rotate <0, 180, 0>
      translate <0, -2.5, 0>
      rotate <-70, 0, 0>
      translate <0, 4.5, 13>
    }
    
    object
    {
      O_LASER_TURRET    
      translate <0, 0, 7>
    }
  }
}
  
#declare O_LASER_CANNON_GIRDER =
object
{
  union
  {
    object
    {
      O_LASER_BARREL
      rotate <0, 180, 0>
      translate <0, -2.5, 0>
      rotate <-70, 0, 0>
      translate <0, 4.5, 13>
    }
    
    object
    {
      O_LASER_TURRET_GIRDER
      translate <0, 0, 7>
    }
  }
}


camera
{
  location <-10, 7, -10>
  look_at <0, 1, 0>
}

light_source
{
  <-10, 15, -5> * 10
  color rgb 1.2
}  

light_source
{
  <10, 7, -2> * 10
  color rgb 0.4
}  

           
#declare TEX_METAL =
texture
{
  pigment 
  {
    color rgb <0.65, 0.63, 0.61>
  }
  
  normal 
  {
    bozo 0.1
    scale 50
  }
  
  finish
  {
    ambient 0
    diffuse 0.7
    specular 0.7
    roughness 0.04
    metallic
    reflection 
    {
      0.1
      metallic
    }
  }    
}

object
{
  O_LASER_CANNON_GIRDER
  translate <0, +10, -7>
  
  scale 0.2
  translate <-2.5, 1.2, 2.5>
  texture { TEX_METAL }
}

object
{
  O_LASER_CANNON_GIRDER
  translate <0, +10, -7>
  
  scale 0.2
  rotate <0, 180, 0>
  translate <2.5, 1.2, -2.5>
  texture { TEX_METAL }
}


object
{
  O_LASER_CANNON
  translate <0, -14, -7>
  
  scale 0.2
  translate <-2.5, 1.2, 2.5>
  texture { TEX_METAL }
}

object
{
  O_LASER_CANNON
  translate <0, -14, -7>
  
  scale 0.2
  rotate <0, 180, 0>
  translate <2.5, 1.2, -2.5>
  texture { TEX_METAL }
}
